@@ -43,7 +43,10 @@ from lava_dashboard_tool.commands import DataSetRenderer, _get_pretty_renderer
 from lava_scheduler_tool.scheduler import (
     devicedictionary_to_jinja2,
     jinja2_to_devicedictionary,
-    validate_urls
+    publish_artifacts,
+    validate_urls,
+    DEFAULT_ARTIFACTS_URL,
+    DEFAULT_ARTIFACTS_UPLOAD_PATH
 )
 from zmq.utils.strtypes import b, u
 
@@ -85,6 +88,18 @@ class submit_job(Command):
         super(submit_job, cls).register_arguments(parser)
         parser.add_argument("SERVER")
         parser.add_argument("JSON_FILE")
+        parser.add_argument(
+            "--publish-artifacts",
+            action="store_true",
+            help="Publish all file:/// references within job definition via publishing API and rewrite URL's.")
+        parser.add_argument(
+            "--artifacts-url",
+            default=DEFAULT_ARTIFACTS_URL,
+            help="Remote server which accepts file uploads via curl.")
+        parser.add_argument(
+            "--artifacts-path",
+            default=DEFAULT_ARTIFACTS_UPLOAD_PATH,
+            help="Location on the remote server to which artifacts will be uploaded.")
         parser.add_argument(
             "--block",
             action="store_true",
@@ -99,6 +114,11 @@ class submit_job(Command):
         with open(self.args.JSON_FILE, 'rb') as stream:
             command_text = stream.read()
         try:
+            if self.args.publish_artifacts:
+                command_text = publish_artifacts(
+                    command_text,
+                    self.args.artifacts_url,
+                    self.args.artifacts_path)
             validate_urls(command_text)
             job_ids = server.scheduler.submit_job(command_text)
         except xmlrpclib.Fault, e:
@@ -625,8 +645,7 @@ class wait_job_events(Command):
     """
 
     FINISHED_JOB_STATUS = ["Complete", "Incomplete", "Canceled"]
-    PUBLISHER_PROTOCOL = "tcp://"
-    PUBLISHER_PORT = 5500
+    EVENT_SOCKET_PATTERN_SYMBOL = "*"
 
     @classmethod
     def register_arguments(cls, parser):
@@ -638,13 +657,22 @@ class wait_job_events(Command):
         action.add_argument("--job-id",
                             type=int,
                             help="ID of the job to wait for. If this argument is provided, this command will not submit new job.")
-        parser.add_argument("--port", "-p",
-                            type=int,
-                            help="Alternative publisher port to use")
         parser.add_argument("--timeout",
                             type=int,
                             default=0,
                             help="Time to wait until the job is finished. Default will wait until manually canceled.")
+        parser.add_argument(
+            "--publish-artifacts",
+            action="store_true",
+            help="Publish all file:/// references within job definition via publishing API and rewrite URL's.")
+        parser.add_argument(
+            "--artifacts-url",
+            default=DEFAULT_ARTIFACTS_URL,
+            help="Remote server which accepts file uploads via curl.")
+        parser.add_argument(
+            "--artifacts-path",
+            default=DEFAULT_ARTIFACTS_UPLOAD_PATH,
+            help="Location on the remote server to which artifacts will be uploaded.")
         parser.add_argument(
             "--quiet", "-q",
             action="store_true",
@@ -656,18 +684,17 @@ class wait_job_events(Command):
             self.args.SERVER, auth_backend=auth_backend)
 
         # Need to get the host in case of shortcuts usage.
-        parsed_host = urlparse.urlparse(
-            get_server_url(auth_backend, self.args.SERVER))
-        host = parsed_host.netloc
+        host = get_server_url(auth_backend, self.args.SERVER)
+        hostname = urlparse.urlparse(host).netloc
 
         if self.args.job_id:
             job_ids = [self.args.job_id]
             try:
-                job_details = server.scheduler.job_details(self.args.job_id)
+                job = server.scheduler.job_details(self.args.job_id)
 
-                if job_details["status"] in self.FINISHED_JOB_STATUS:
+                if job["status"] in self.FINISHED_JOB_STATUS:
                     print "Job %s already finished with status %s." % (
-                        self.args.job_id, job_details["status"])
+                        self.args.job_id, job["status"])
                     return
 
             except xmlrpclib.Fault as e:
@@ -684,17 +711,31 @@ class wait_job_events(Command):
                 print >> sys.stderr, e
                 return -1
             try:
+                if self.args.publish_artifacts:
+                    command_text = publish_artifacts(
+                        command_text,
+                        self.args.artifacts_url,
+                        self.args.artifacts_path)
                 validate_urls(command_text)
                 job_ids = server.scheduler.submit_job(command_text)
-                print "submitted as job id(s):", job_ids
+
                 if not isinstance(job_ids, list):
                     job_ids = [job_ids]
 
+                print "submitted as job(s):"
+                for job_id in job_ids:
+                    job = server.scheduler.job_details(job_id)
+                    print urlparse.urljoin(host, job["absolute_url"])
+
             except xmlrpclib.Fault, e:
                 raise CommandError(str(e))
 
-        port = self.args.port if self.args.port else self.PUBLISHER_PORT
-        event_socket = "%s%s:%s" % (self.PUBLISHER_PROTOCOL, host, port)
+        job_url_template = urlparse.urljoin(
+            host, job["absolute_url"].rsplit("/", 1)[0])
+
+        host_pattern = server.scheduler.get_publisher_event_socket()
+        event_socket = host_pattern.replace(
+            self.EVENT_SOCKET_PATTERN_SYMBOL, hostname, 1)
 
         try:
             with Timeout(self.args.timeout):
@@ -704,7 +745,7 @@ class wait_job_events(Command):
                 sock.setsockopt(zmq.SUBSCRIBE, b"")
                 sock.connect(event_socket)
 
-                print >> sys.stdout, "Now waiting for job events..."
+                print >> sys.stdout, "Now waiting for job events...\n"
                 while True:
                     msg = sock.recv_multipart()
                     try:
@@ -718,11 +759,13 @@ class wait_job_events(Command):
                         # Filter out device events.
                         if "submitter" in data:
                             if data["job"] in job_ids:
+                                data["job"] = urlparse.urljoin(
+                                    job_url_template, str(data["job"]))
                                 if data["status"] in self.FINISHED_JOB_STATUS:
-                                    print data
+                                    print data, "\n"
                                     break
                                 elif not self.args.quiet:
-                                    print data
+                                    print data, "\n"
 
         except zmq.error.ZMQError as e:
             print >> sys.stderr, "wait_job_events() error: %s" % e
@@ -770,3 +813,112 @@ class validate_pipeline_devices(Command):
                 self.args.name)
         except xmlrpclib.Fault, e:
             raise CommandError(str(e))
+
+
+class job_results(Command):
+    """
+    Get job results as CSV or YAML.
+    """
+
+    @classmethod
+    def register_arguments(cls, parser):
+        super(job_results, cls).register_arguments(parser)
+        parser.add_argument("SERVER",
+                            help="Host from where to download the results.")
+        parser.add_argument("JOB_ID",
+                            type=int,
+                            help="ID of the job to get results for.")
+        parser.add_argument("--csv",
+                            action="store_true",
+                            help="If this is specified, results will be printed out in CSV format, otherwise in YAML.")
+
+    def invoke(self):
+        auth_backend = KeyringAuthBackend()
+        server = AuthenticatingServerProxy(
+            self.args.SERVER, auth_backend=auth_backend)
+
+        try:
+            if self.args.csv:
+                print server.results.get_testjob_results_csv(self.args.JOB_ID)
+            else:
+                print server.results.get_testjob_results_yaml(self.args.JOB_ID)
+        except xmlrpclib.Fault as e:
+            print "Command did not found specified job. Please make sure JOB_ID exists on the specified SERVER."
+
+
+class test_suite_results(Command):
+    """
+    Get test suite results as CSV or YAML.
+    """
+
+    @classmethod
+    def register_arguments(cls, parser):
+        super(test_suite_results, cls).register_arguments(parser)
+        parser.add_argument("SERVER",
+                            help="Host from where to download the results.")
+        parser.add_argument("JOB_ID",
+                            type=int,
+                            help="ID of the job to get results from.")
+        parser.add_argument("SUITE_NAME",
+                            help="Name of the suite to get results for.")
+        parser.add_argument("--csv",
+                            action="store_true",
+                            help="If this is specified, results will be printed out in CSV format, otherwise in YAML.")
+
+    def invoke(self):
+        auth_backend = KeyringAuthBackend()
+        server = AuthenticatingServerProxy(
+            self.args.SERVER, auth_backend=auth_backend)
+
+        try:
+            if self.args.csv:
+                print server.results.get_testsuite_results_csv(
+                    self.args.JOB_ID,
+                    self.args.SUITE_NAME)
+            else:
+                print server.results.get_testsuite_results_yaml(
+                    self.args.JOB_ID,
+                    self.args.SUITE_NAME)
+        except xmlrpclib.Fault as e:
+            print "Command did not found specified job or test suite. Please make sure JOB_ID and SUITE_NAME exist on the specified SERVER."
+
+
+class test_case_results(Command):
+    """
+    Get test case results as CSV or YAML.
+    """
+
+    @classmethod
+    def register_arguments(cls, parser):
+        super(test_case_results, cls).register_arguments(parser)
+        parser.add_argument("SERVER",
+                            help="Host from where to download the results.")
+        parser.add_argument("JOB_ID",
+                            type=int,
+                            help="ID of the job to get results from.")
+        parser.add_argument("SUITE_NAME",
+                            help="Name of the suite to get results from.")
+        parser.add_argument("TEST_CASE",
+                            help="Name of the test case to get results for.")
+        parser.add_argument("--csv",
+                            action="store_true",
+                            help="If this is specified, results will be printed out in CSV format, otherwise in YAML.")
+
+    def invoke(self):
+        auth_backend = KeyringAuthBackend()
+        server = AuthenticatingServerProxy(
+            self.args.SERVER, auth_backend=auth_backend)
+
+        try:
+            if self.args.csv:
+                print server.results.get_testcase_results_csv(
+                    self.args.JOB_ID,
+                    self.args.SUITE_NAME,
+                    self.args.TEST_CASE)
+            else:
+                print server.results.get_testcase_results_yaml(
+                    self.args.JOB_ID,
+                    self.args.SUITE_NAME,
+                    self.args.TEST_CASE)
+        except xmlrpclib.Fault as e:
+            print "Command did not found specified job, test suite or test case. Please make sure JOB_ID, SUITE_NAME and TEST_CASE exist on the specified SERVER."
